// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "SubSceneController.hpp"

using namespace Dawn;

SubSceneController::SubSceneController(SceneItem *i)  : SceneItemComponent(i) {

}

void SubSceneController::onSceneUpdate() {
  if(this->onlyUpdateUnpaused) return;
  if(this->subScene == nullptr) return;
  this->subScene->update();
}

void SubSceneController::onSceneUnpausedUpdate() {
  if(!this->onlyUpdateUnpaused) return;
  if(this->subScene == nullptr) return;
  this->subScene->update();
}

Scene * SubSceneController::getSubScene() {
  return this->subScene;
}

void SubSceneController::setSubScene(Scene *scene) {
  assertTrue(scene != this->subScene);
  this->subScene = scene;
}

void SubSceneController::onStart() {
  auto myScene = this->getScene();
  myScene->eventSceneUnpausedUpdate.addListener(this, &SubSceneController::onSceneUnpausedUpdate);
  myScene->eventSceneUpdate.addListener(this, &SubSceneController::onSceneUpdate);
}

void SubSceneController::onDispose() {
  auto myScene = this->getScene();
  myScene->eventSceneUnpausedUpdate.removeListener(this, &SubSceneController::onSceneUnpausedUpdate);
  myScene->eventSceneUpdate.removeListener(this, &SubSceneController::onSceneUpdate);
}